/* Complex Plot
 * ------------
 *
 * Calculating f(z) for each pixels,
 * generating the picture using "my argument"
 * which is much faster than the original one in MAPMX.
 * Based on complex.h.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <complex.h>

#include "resolution.h"

#define XINC ((XMAX-XMIN)/SIZE)
#define YINC ((YMAX-YMIN)/SIZE)

#ifdef _DMALLOC_
#include <dmalloc.h>
#endif

#define BUFFSIZE 100

clock_t start;
long totaltime = 0;
int R = 0, G = 0, B = 0;

/* This code is based on Zoltan Kovacs's idea */

#define MYPI 3.14159

double
val (double x, double y, double h)
{
  double hh;
  hh = h;
  if (h > 360.0)
    hh -= 360.00;
  else if (h < 0.0)
    hh += 360.0;
  if (hh < 60.0)
    return (x + (y - x) * hh / 60.0);
  else if (hh < 180.0)
    return y;
  else if (hh < 240.0)
    return (x + (y - x) * (240.0 - hh) / 60.0);
  return x;
}

int
col (double x)
{
  int code;
  code = (int) (x * 256);
  if (code > 255)
    return 255;
  else
    return (code);
}

void
hlstorgb (double h, double l, double s)
{
  double m1, m2;
  if (l < 0.5)
    m2 = l * (1 + s);
  else
    m2 = l + s - l * s;
  m1 = 2.0 * l - m2;
  R = col (val (m1, m2, h + 120.0));
  G = col (val (m1, m2, h));
  B = col (val (m1, m2, h - 120.0));
}

double
myargument (double zre, double zim)
{
  double fcarg, nograd;

  // My argument, instead of MAPMX:
  if (zre == 0)
    {
      if (zim == 0)
	fcarg = 0;
      else
	{
	  fcarg = MYPI / 2.0;
	  if (zim < 0)
	    fcarg = -fcarg;
	}
    }
  else
    {
      fcarg = atan (zim / zre);
      if (zre < 0)
	fcarg += MYPI;
    }
  // End of my argument.

  if (fcarg < 0.0)
    nograd = (fcarg + 2 * MYPI) / MYPI * 180.0;
  else
    nograd = fcarg / MYPI * 180.0;
  return (nograd);
}

double
smax (double x)
{
  if (x <= 0.5)
    return 2 * x;
  else
    return (2 - 2 * x);
}

void
rgbcolor (double zre, double zim)
{
  double xx;
  xx = 1.0 - 2.0 * atan (sqrt (zre * zre + zim * zim)) / MYPI;
  hlstorgb (myargument (zre, zim), xx, smax (xx));
}

void
timer_start ()
{
  start = clock ();
}

void
timer_end ()
{
  clock_t elapsed = clock () - start;
  printf ("\r%d", totaltime);
  totaltime += elapsed;
}

complex
complex_function (complex z)
{
  return
#include "function.c"
    ;
}

int
main (void)
{
  double i, j;
  int i_, j_;
  complex z;

  printf ("P3\n%d %d\n255\n", (int) SIZE + 1, (int) SIZE + 1);

  for (i_ = 0, i = YMIN; i_ <= SIZE; i += YINC, i_++)
    {
      for (j_ = 0, j = XMIN; j_ <= SIZE; j += XINC, j_++)
	{
	  z = complex_function (j + i * I);
	  rgbcolor (creal (z), cimag (z));
	  printf ("%d %d %d\n", R, G, B);
	}
    }

  return 0;
}
